/**
 * Copyright (C) 2010 Michael Mosmann <michael@mosmann.de>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.flapdoodle.mongoom.parser.properties;

import java.lang.annotation.Annotation;
import java.lang.reflect.Type;

import de.flapdoodle.mongoom.parser.IType;

public class ClassType implements IType {

	private final Class<?> _clazz;

	private ClassType(Class<?> clazz) {
		_clazz = clazz;
	}

	@Override
	public Type getGenericType() {
		return null;
	}

	@Override
	public Class<?> getType() {
		return _clazz;
	}
	
	@Override
	public <T extends Annotation> T getAnnotation(Class<T> annotationClass) {
		return _clazz.getAnnotation(annotationClass);
	}
	
	public static ClassType of(Class<?> clazz) {
		return new ClassType(clazz);
	}
	
	@Override
	public String toString() {
		return "ClassType("+_clazz+")";
	}

}
